@@ -1,11 +1,15 @@
+import logging
 import threading
 from numbers import Number
-from typing import Dict
+from typing import Dict, Optional
 
 from py4j.java_collections import JavaIterator
 from py4j.java_gateway import JavaObject
+from pyspark.java_gateway import ensure_callback_server_started
 from pyspark.sql import SparkSession
 
+logger = logging.getLogger(__name__)
+
 
 # noinspection PyPep8Naming
 class Listener:
@@ -33,20 +37,18 @@ class Listener:
 
 # noinspection PyProtectedMember
 class SparkMetrics:
-    def __init__(self, spark: SparkSession, timeout: float = 15):
+    def __init__(self, spark: SparkSession, timeout: Optional[float] = 15):
         self._spark = spark
         self._semaphore = threading.Semaphore(0)
         self._listener = Listener(spark.sparkContext._gateway, self._semaphore)
         self._timeout = timeout
 
     def __enter__(self) -> Dict[str, Number]:
-        gateway = self._spark.sparkContext._gateway
-        gateway.callback_server_parameters.daemonize = True
-        gateway.callback_server_parameters.daemonize_connections = True
-        gateway.start_callback_server()
+        ensure_callback_server_started(self._spark.sparkContext._gateway)
         self._spark._jsparkSession.listenerManager().register(self._listener)
         return self._listener._metrics
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        self._semaphore.acquire(timeout=self._timeout)
+        if not self._semaphore.acquire(timeout=self._timeout):
+            logger.warning("Did not receive any metrics within %s seconds", self._timeout)
         self._spark._jsparkSession.listenerManager().unregister(self._listener)
